@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's symbolic exponentiation (basis^exponent). */
 
 /*
- *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -140,7 +140,7 @@ void power::do_print_latex(const print_latex & c, unsigned level) const
 static void print_sym_pow(const print_context & c, const symbol &x, int exp)
 {
 	// Optimal output of integer powers of symbols to aid compiler CSE.
-	// C.f. ISO/IEC 14882:1998, section 1.9 [intro execution], paragraph 15
+	// C.f. ISO/IEC 14882:2011, section 1.9 [intro execution], paragraph 15
 	// to learn why such a parenthesation is really necessary.
 	if (exp == 1) {
 		x.print(c);
@@ -229,21 +229,18 @@ bool power::info(unsigned inf) const
 		case info_flags::cinteger_polynomial:
 		case info_flags::rational_polynomial:
 		case info_flags::crational_polynomial:
-			return exponent.info(info_flags::nonnegint) &&
-			       basis.info(inf);
+			return basis.info(inf) && exponent.info(info_flags::nonnegint);
 		case info_flags::rational_function:
-			return exponent.info(info_flags::integer) &&
-			       basis.info(inf);
-		case info_flags::algebraic:
-			return !exponent.info(info_flags::integer) ||
-			       basis.info(inf);
+			return basis.info(inf) && exponent.info(info_flags::integer);
+		case info_flags::real:
+			return basis.info(inf) && exponent.info(info_flags::integer);
 		case info_flags::expanded:
 			return (flags & status_flags::expanded);
 		case info_flags::positive:
 			return basis.info(info_flags::positive) && exponent.info(info_flags::real);
 		case info_flags::nonnegative:
 			return (basis.info(info_flags::positive) && exponent.info(info_flags::real)) ||
-			       (basis.info(info_flags::real) && exponent.info(info_flags::integer) && exponent.info(info_flags::even));
+			       (basis.info(info_flags::real) && exponent.info(info_flags::even));
 		case info_flags::has_indices: {
 			if (flags & status_flags::has_indices)
 				return true;
@@ -282,7 +279,7 @@ ex power::map(map_function & f) const
 
 	if (!are_ex_trivially_equal(basis, mapped_basis)
 	 || !are_ex_trivially_equal(exponent, mapped_exponent))
-		return (new power(mapped_basis, mapped_exponent))->setflag(status_flags::dynallocated);
+		return dynallocate<power>(mapped_basis, mapped_exponent);
 	else
 		return *this;
 }
@@ -373,42 +370,36 @@ ex power::coeff(const ex & s, int n) const
  *  - ^(*(x,y,z),c) -> *(x^c,y^c,z^c)  (if c integer)
  *  - ^(*(x,c1),c2) -> ^(x,c2)*c1^c2  (c1>0)
  *  - ^(*(x,c1),c2) -> ^(-x,c2)*c1^c2  (c1<0)
- *
- *  @param level cut-off in recursive evaluation */
-ex power::eval(int level) const
+ */
+ex power::eval() const
 {
-	if ((level==1) && (flags & status_flags::evaluated))
+	if (flags & status_flags::evaluated)
 		return *this;
-	else if (level == -max_recursion_level)
-		throw(std::runtime_error("max recursion level reached"));
-	
-	const ex & ebasis    = level==1 ? basis    : basis.eval(level-1);
-	const ex & eexponent = level==1 ? exponent : exponent.eval(level-1);
-	
-	const numeric *num_basis = NULL;
-	const numeric *num_exponent = NULL;
-	
-	if (is_exactly_a<numeric>(ebasis)) {
-		num_basis = &ex_to<numeric>(ebasis);
+
+	const numeric *num_basis = nullptr;
+	const numeric *num_exponent = nullptr;
+
+	if (is_exactly_a<numeric>(basis)) {
+		num_basis = &ex_to<numeric>(basis);
 	}
-	if (is_exactly_a<numeric>(eexponent)) {
-		num_exponent = &ex_to<numeric>(eexponent);
+	if (is_exactly_a<numeric>(exponent)) {
+		num_exponent = &ex_to<numeric>(exponent);
 	}
 	
 	// ^(x,0) -> 1  (0^0 also handled here)
-	if (eexponent.is_zero()) {
-		if (ebasis.is_zero())
+	if (exponent.is_zero()) {
+		if (basis.is_zero())
 			throw (std::domain_error("power::eval(): pow(0,0) is undefined"));
 		else
 			return _ex1;
 	}
 	
 	// ^(x,1) -> x
-	if (eexponent.is_equal(_ex1))
-		return ebasis;
+	if (exponent.is_equal(_ex1))
+		return basis;
 
 	// ^(0,c1) -> 0 or exception  (depending on real value of c1)
-	if ( ebasis.is_zero() && num_exponent ) {
+	if (basis.is_zero() && num_exponent) {
 		if ((num_exponent->real()).is_zero())
 			throw (std::domain_error("power::eval(): pow(0,I) is undefined"));
 		else if ((num_exponent->real()).is_negative())
@@ -418,16 +409,16 @@ ex power::eval(int level) const
 	}
 
 	// ^(1,x) -> 1
-	if (ebasis.is_equal(_ex1))
+	if (basis.is_equal(_ex1))
 		return _ex1;
 
 	// power of a function calculated by separate rules defined for this function
-	if (is_exactly_a<function>(ebasis))
-		return ex_to<function>(ebasis).power(eexponent);
+	if (is_exactly_a<function>(basis))
+		return ex_to<function>(basis).power(exponent);
 
 	// Turn (x^c)^d into x^(c*d) in the case that x is positive and c is real.
-	if (is_exactly_a<power>(ebasis) && ebasis.op(0).info(info_flags::positive) && ebasis.op(1).info(info_flags::real))
-		return power(ebasis.op(0), ebasis.op(1) * eexponent);
+	if (is_exactly_a<power>(basis) && basis.op(0).info(info_flags::positive) && basis.op(1).info(info_flags::real))
+		return dynallocate<power>(basis.op(0), basis.op(1) * exponent);
 
 	if ( num_exponent ) {
 
@@ -438,9 +429,7 @@ ex power::eval(int level) const
 			const bool exponent_is_crational = num_exponent->is_crational();
 			if (!basis_is_crational || !exponent_is_crational) {
 				// return a plain float
-				return (new numeric(num_basis->power(*num_exponent)))->setflag(status_flags::dynallocated |
-				                                                               status_flags::evaluated |
-				                                                               status_flags::expanded);
+				return dynallocate<numeric>(num_basis->power(*num_exponent));
 			}
 
 			const numeric res = num_basis->power(*num_exponent);
@@ -470,9 +459,9 @@ ex power::eval(int level) const
 						const numeric res_bnum = bnum.power(*num_exponent);
 						const numeric res_bden = bden.power(*num_exponent);
 						if (res_bnum.is_integer())
-							return (new mul(power(bden,-*num_exponent),res_bnum))->setflag(status_flags::dynallocated | status_flags::evaluated);
+							return dynallocate<mul>(dynallocate<power>(bden,-*num_exponent),res_bnum).setflag(status_flags::evaluated);
 						if (res_bden.is_integer())
-							return (new mul(power(bnum,*num_exponent),res_bden.inverse()))->setflag(status_flags::dynallocated | status_flags::evaluated);
+							return dynallocate<mul>(dynallocate<power>(bnum,*num_exponent),res_bden.inverse()).setflag(status_flags::evaluated);
 					}
 					return this->hold();
 				} else {
@@ -480,8 +469,7 @@ ex power::eval(int level) const
 					// because otherwise we'll end up with something like
 					//    (7/8)^(4/3)  ->  7/8*(1/2*7^(1/3))
 					// instead of 7/16*7^(1/3).
-					ex prod = power(*num_basis,r.div(m));
-					return prod*power(*num_basis,q);
+					return pow(basis, r.div(m)) * pow(basis, q);
 				}
 			}
 		}
@@ -489,8 +477,8 @@ ex power::eval(int level) const
 		// ^(^(x,c1),c2) -> ^(x,c1*c2)
 		// (c1, c2 numeric(), c2 integer or -1 < c1 <= 1 or (c1=-1 and c2>0),
 		// case c1==1 should not happen, see below!)
-		if (is_exactly_a<power>(ebasis)) {
-			const power & sub_power = ex_to<power>(ebasis);
+		if (is_exactly_a<power>(basis)) {
+			const power & sub_power = ex_to<power>(basis);
 			const ex & sub_basis = sub_power.basis;
 			const ex & sub_exponent = sub_power.exponent;
 			if (is_exactly_a<numeric>(sub_exponent)) {
@@ -498,21 +486,21 @@ ex power::eval(int level) const
 				GINAC_ASSERT(num_sub_exponent!=numeric(1));
 				if (num_exponent->is_integer() || (abs(num_sub_exponent) - (*_num1_p)).is_negative() ||
 				    (num_sub_exponent == *_num_1_p && num_exponent->is_positive())) {
-					return power(sub_basis,num_sub_exponent.mul(*num_exponent));
+					return dynallocate<power>(sub_basis, num_sub_exponent.mul(*num_exponent));
 				}
 			}
 		}
 	
 		// ^(*(x,y,z),c1) -> *(x^c1,y^c1,z^c1) (c1 integer)
-		if (num_exponent->is_integer() && is_exactly_a<mul>(ebasis)) {
-			return expand_mul(ex_to<mul>(ebasis), *num_exponent, 0);
+		if (num_exponent->is_integer() && is_exactly_a<mul>(basis)) {
+			return expand_mul(ex_to<mul>(basis), *num_exponent, false);
 		}
 
 		// (2*x + 6*y)^(-4) -> 1/16*(x + 3*y)^(-4)
-		if (num_exponent->is_integer() && is_exactly_a<add>(ebasis)) {
-			numeric icont = ebasis.integer_content();
+		if (num_exponent->is_integer() && is_exactly_a<add>(basis)) {
+			numeric icont = basis.integer_content();
 			const numeric lead_coeff = 
-				ex_to<numeric>(ex_to<add>(ebasis).seq.begin()->coeff).div(icont);
+				ex_to<numeric>(ex_to<add>(basis).seq.begin()->coeff).div(icont);
 
 			const bool canonicalizable = lead_coeff.is_integer();
 			const bool unit_normal = lead_coeff.is_pos_integer();
@@ -520,48 +508,43 @@ ex power::eval(int level) const
 				icont = icont.mul(*_num_1_p);
 			
 			if (canonicalizable && (icont != *_num1_p)) {
-				const add& addref = ex_to<add>(ebasis);
-				add* addp = new add(addref);
-				addp->setflag(status_flags::dynallocated);
-				addp->clearflag(status_flags::hash_calculated);
-				addp->overall_coeff = ex_to<numeric>(addp->overall_coeff).div_dyn(icont);
-				for (epvector::iterator i = addp->seq.begin(); i != addp->seq.end(); ++i)
-					i->coeff = ex_to<numeric>(i->coeff).div_dyn(icont);
+				const add& addref = ex_to<add>(basis);
+				add & addp = dynallocate<add>(addref);
+				addp.clearflag(status_flags::hash_calculated);
+				addp.overall_coeff = ex_to<numeric>(addp.overall_coeff).div_dyn(icont);
+				for (auto & i : addp.seq)
+					i.coeff = ex_to<numeric>(i.coeff).div_dyn(icont);
 
 				const numeric c = icont.power(*num_exponent);
 				if (likely(c != *_num1_p))
-					return (new mul(power(*addp, *num_exponent), c))->setflag(status_flags::dynallocated);
+					return dynallocate<mul>(dynallocate<power>(addp, *num_exponent), c);
 				else
-					return power(*addp, *num_exponent);
+					return dynallocate<power>(addp, *num_exponent);
 			}
 		}
 
 		// ^(*(...,x;c1),c2) -> *(^(*(...,x;1),c2),c1^c2)  (c1, c2 numeric(), c1>0)
 		// ^(*(...,x;c1),c2) -> *(^(*(...,x;-1),c2),(-c1)^c2)  (c1, c2 numeric(), c1<0)
-		if (is_exactly_a<mul>(ebasis)) {
+		if (is_exactly_a<mul>(basis)) {
 			GINAC_ASSERT(!num_exponent->is_integer()); // should have been handled above
-			const mul & mulref = ex_to<mul>(ebasis);
+			const mul & mulref = ex_to<mul>(basis);
 			if (!mulref.overall_coeff.is_equal(_ex1)) {
 				const numeric & num_coeff = ex_to<numeric>(mulref.overall_coeff);
 				if (num_coeff.is_real()) {
 					if (num_coeff.is_positive()) {
-						mul *mulp = new mul(mulref);
-						mulp->overall_coeff = _ex1;
-						mulp->setflag(status_flags::dynallocated);
-						mulp->clearflag(status_flags::evaluated);
-						mulp->clearflag(status_flags::hash_calculated);
-						return (new mul(power(*mulp,exponent),
-						                power(num_coeff,*num_exponent)))->setflag(status_flags::dynallocated);
+						mul & mulp = dynallocate<mul>(mulref);
+						mulp.overall_coeff = _ex1;
+						mulp.clearflag(status_flags::evaluated | status_flags::hash_calculated);
+						return dynallocate<mul>(dynallocate<power>(mulp, exponent),
+						                        dynallocate<power>(num_coeff, *num_exponent));
 					} else {
 						GINAC_ASSERT(num_coeff.compare(*_num0_p)<0);
 						if (!num_coeff.is_equal(*_num_1_p)) {
-							mul *mulp = new mul(mulref);
-							mulp->overall_coeff = _ex_1;
-							mulp->setflag(status_flags::dynallocated);
-							mulp->clearflag(status_flags::evaluated);
-							mulp->clearflag(status_flags::hash_calculated);
-							return (new mul(power(*mulp,exponent),
-							                power(abs(num_coeff),*num_exponent)))->setflag(status_flags::dynallocated);
+							mul & mulp = dynallocate<mul>(mulref);
+							mulp.overall_coeff = _ex_1;
+							mulp.clearflag(status_flags::evaluated | status_flags::hash_calculated);
+							return dynallocate<mul>(dynallocate<power>(mulp, exponent),
+							                        dynallocate<power>(abs(num_coeff), *num_exponent));
 						}
 					}
 				}
@@ -570,39 +553,26 @@ ex power::eval(int level) const
 
 		// ^(nc,c1) -> ncmul(nc,nc,...) (c1 positive integer, unless nc is a matrix)
 		if (num_exponent->is_pos_integer() &&
-		    ebasis.return_type() != return_types::commutative &&
-		    !is_a<matrix>(ebasis)) {
-			return ncmul(exvector(num_exponent->to_int(), ebasis), true);
+		    basis.return_type() != return_types::commutative &&
+		    !is_a<matrix>(basis)) {
+			return ncmul(exvector(num_exponent->to_int(), basis));
 		}
 	}
-	
-	if (are_ex_trivially_equal(ebasis,basis) &&
-	    are_ex_trivially_equal(eexponent,exponent)) {
-		return this->hold();
-	}
-	return (new power(ebasis, eexponent))->setflag(status_flags::dynallocated |
-	                                               status_flags::evaluated);
+
+	return this->hold();
 }
 
-ex power::evalf(int level) const
+ex power::evalf() const
 {
-	ex ebasis;
+	ex ebasis = basis.evalf();
 	ex eexponent;
 	
-	if (level==1) {
-		ebasis = basis;
+	if (!is_exactly_a<numeric>(exponent))
+		eexponent = exponent.evalf();
+	else
 		eexponent = exponent;
-	} else if (level == -max_recursion_level) {
-		throw(std::runtime_error("max recursion level reached"));
-	} else {
-		ebasis = basis.evalf(level-1);
-		if (!is_exactly_a<numeric>(exponent))
-			eexponent = exponent.evalf(level-1);
-		else
-			eexponent = exponent;
-	}
 
-	return power(ebasis,eexponent);
+	return dynallocate<power>(ebasis, eexponent);
 }
 
 ex power::evalm() const
@@ -611,10 +581,10 @@ ex power::evalm() const
 	const ex eexponent = exponent.evalm();
 	if (is_a<matrix>(ebasis)) {
 		if (is_exactly_a<numeric>(eexponent)) {
-			return (new matrix(ex_to<matrix>(ebasis).pow(eexponent)))->setflag(status_flags::dynallocated);
+			return dynallocate<matrix>(ex_to<matrix>(ebasis).pow(eexponent));
 		}
 	}
-	return (new power(ebasis, eexponent))->setflag(status_flags::dynallocated);
+	return dynallocate<power>(ebasis, eexponent);
 }
 
 bool power::has(const ex & other, unsigned options) const
@@ -654,13 +624,13 @@ ex power::subs(const exmap & m, unsigned options) const
 	if (!(options & subs_options::algebraic))
 		return subs_one_level(m, options);
 
-	for (exmap::const_iterator it = m.begin(); it != m.end(); ++it) {
+	for (auto & it : m) {
 		int nummatches = std::numeric_limits<int>::max();
 		exmap repls;
-		if (tryfactsubs(*this, it->first, nummatches, repls)) {
-			ex anum = it->second.subs(repls, subs_options::no_pattern);
-			ex aden = it->first.subs(repls, subs_options::no_pattern);
-			ex result = (*this)*power(anum/aden, nummatches);
+		if (tryfactsubs(*this, it.first, nummatches, repls)) {
+			ex anum = it.second.subs(repls, subs_options::no_pattern);
+			ex aden = it.first.subs(repls, subs_options::no_pattern);
+			ex result = (*this) * pow(anum/aden, nummatches);
 			return (ex_to<basic>(result)).subs_one_level(m, options);
 		}
 	}
@@ -682,14 +652,14 @@ ex power::conjugate() const
 		if (are_ex_trivially_equal(exponent, newexponent)) {
 			return *this;
 		}
-		return (new power(basis, newexponent))->setflag(status_flags::dynallocated);
+		return dynallocate<power>(basis, newexponent);
 	}
 	if (exponent.info(info_flags::integer)) {
 		ex newbasis = basis.conjugate();
 		if (are_ex_trivially_equal(basis, newbasis)) {
 			return *this;
 		}
-		return (new power(newbasis, exponent))->setflag(status_flags::dynallocated);
+		return dynallocate<power>(newbasis, exponent);
 	}
 	return conjugate_function(*this).hold();
 }
@@ -699,7 +669,8 @@ ex power::real_part() const
 	// basis == a+I*b, exponent == c+I*d
 	const ex a = basis.real_part();
 	const ex c = exponent.real_part();
-	if (basis.is_equal(a) && exponent.is_equal(c)) {
+	if (basis.is_equal(a) && exponent.is_equal(c) &&
+	    (a.info(info_flags::nonnegative) || c.info(info_flags::integer))) {
 		// Re(a^c)
 		return *this;
 	}
@@ -709,12 +680,12 @@ ex power::real_part() const
 		// Re((a+I*b)^c)  w/  c ∈ ℤ
 		long N = ex_to<numeric>(c).to_long();
 		// Use real terms in Binomial expansion to construct
-		// Re(expand(power(a+I*b, N))).
+		// Re(expand(pow(a+I*b, N))).
 		long NN = N > 0 ? N : -N;
-		ex numer = N > 0 ? _ex1 : power(power(a,2) + power(b,2), NN);
+		ex numer = N > 0 ? _ex1 : pow(pow(a,2) + pow(b,2), NN);
 		ex result = 0;
 		for (long n = 0; n <= NN; n += 2) {
-			ex term = binomial(NN, n) * power(a, NN-n) * power(b, n) / numer;
+			ex term = binomial(NN, n) * pow(a, NN-n) * pow(b, n) / numer;
 			if (n % 4 == 0) {
 				result += term;  // sign: I^n w/ n == 4*m
 			} else {
@@ -726,14 +697,16 @@ ex power::real_part() const
 
 	// Re((a+I*b)^(c+I*d))
 	const ex d = exponent.imag_part();
-	return power(abs(basis),c)*exp(-d*atan2(b,a))*cos(c*atan2(b,a)+d*log(abs(basis)));
+	return pow(abs(basis),c) * exp(-d*atan2(b,a)) * cos(c*atan2(b,a)+d*log(abs(basis)));
 }
 
 ex power::imag_part() const
 {
+	// basis == a+I*b, exponent == c+I*d
 	const ex a = basis.real_part();
 	const ex c = exponent.real_part();
-	if (basis.is_equal(a) && exponent.is_equal(c)) {
+	if (basis.is_equal(a) && exponent.is_equal(c) &&
+	    (a.info(info_flags::nonnegative) || c.info(info_flags::integer))) {
 		// Im(a^c)
 		return 0;
 	}
@@ -743,13 +716,13 @@ ex power::imag_part() const
 		// Im((a+I*b)^c)  w/  c ∈ ℤ
 		long N = ex_to<numeric>(c).to_long();
 		// Use imaginary terms in Binomial expansion to construct
-		// Im(expand(power(a+I*b, N))).
+		// Im(expand(pow(a+I*b, N))).
 		long p = N > 0 ? 1 : 3;  // modulus for positive sign
 		long NN = N > 0 ? N : -N;
-		ex numer = N > 0 ? _ex1 : power(power(a,2) + power(b,2), NN);
+		ex numer = N > 0 ? _ex1 : pow(pow(a,2) + pow(b,2), NN);
 		ex result = 0;
 		for (long n = 1; n <= NN; n += 2) {
-			ex term = binomial(NN, n) * power(a, NN-n) * power(b, n) / numer;
+			ex term = binomial(NN, n) * pow(a, NN-n) * pow(b, n) / numer;
 			if (n % 4 == p) {
 				result += term;  // sign: I^n w/ n == 4*m+p
 			} else {
@@ -761,7 +734,7 @@ ex power::imag_part() const
 
 	// Im((a+I*b)^(c+I*d))
 	const ex d = exponent.imag_part();
-	return power(abs(basis),c)*exp(-d*atan2(b,a))*sin(c*atan2(b,a)+d*log(abs(basis)));
+	return pow(abs(basis),c) * exp(-d*atan2(b,a)) * sin(c*atan2(b,a)+d*log(abs(basis)));
 }
 
 // protected
@@ -772,16 +745,11 @@ ex power::derivative(const symbol & s) const
 {
 	if (is_a<numeric>(exponent)) {
 		// D(b^r) = r * b^(r-1) * D(b) (faster than the formula below)
-		epvector newseq;
-		newseq.reserve(2);
-		newseq.push_back(expair(basis, exponent - _ex1));
-		newseq.push_back(expair(basis.diff(s), _ex1));
-		return mul(newseq, exponent);
+		const epvector newseq = {expair(basis, exponent - _ex1), expair(basis.diff(s), _ex1)};
+		return dynallocate<mul>(std::move(newseq), exponent);
 	} else {
 		// D(b^e) = b^e * (D(e)*ln(b) + e*D(b)/b)
-		return mul(*this,
-		           add(mul(exponent.diff(s), log(basis)),
-		           mul(mul(exponent, basis.diff(s)), power(basis, _ex_1))));
+		return *this * (exponent.diff(s)*log(basis) + exponent*basis.diff(s)*pow(basis, _ex_1));
 	}
 }
 
@@ -823,29 +791,26 @@ ex power::expand(unsigned options) const
 		epvector powseq;
 		prodseq.reserve(m.seq.size() + 1);
 		powseq.reserve(m.seq.size() + 1);
-		epvector::const_iterator last = m.seq.end();
-		epvector::const_iterator cit = m.seq.begin();
 		bool possign = true;
 
 		// search for positive/negative factors
-		while (cit!=last) {
-			ex e=m.recombine_pair_to_ex(*cit);
+		for (auto & cit : m.seq) {
+			ex e=m.recombine_pair_to_ex(cit);
 			if (e.info(info_flags::positive))
 				prodseq.push_back(pow(e, exponent).expand(options));
 			else if (e.info(info_flags::negative)) {
 				prodseq.push_back(pow(-e, exponent).expand(options));
 				possign = !possign;
 			} else
-				powseq.push_back(*cit);
-			++cit;
+				powseq.push_back(cit);
 		}
 
 		// take care on the numeric coefficient
 		ex coeff=(possign? _ex1 : _ex_1);
 		if (m.overall_coeff.info(info_flags::positive) && m.overall_coeff != _ex1)
-			prodseq.push_back(power(m.overall_coeff, exponent));
+			prodseq.push_back(pow(m.overall_coeff, exponent));
 		else if (m.overall_coeff.info(info_flags::negative) && m.overall_coeff != _ex_1)
-			prodseq.push_back(power(-m.overall_coeff, exponent));
+			prodseq.push_back(pow(-m.overall_coeff, exponent));
 		else
 			coeff *= m.overall_coeff;
 
@@ -853,9 +818,9 @@ ex power::expand(unsigned options) const
 		// In either case we set a flag to avoid the second run on a part
 		// which does not have positive/negative terms.
 		if (prodseq.size() > 0) {
-			ex newbasis = coeff*mul(powseq);
+			ex newbasis = dynallocate<mul>(std::move(powseq), coeff);
 			ex_to<basic>(newbasis).setflag(status_flags::purely_indefinite);
-			return ((new mul(prodseq))->setflag(status_flags::dynallocated)*(new power(newbasis, exponent))->setflag(status_flags::dynallocated).expand(options)).expand(options);
+			return dynallocate<mul>(std::move(prodseq)) * pow(newbasis, exponent);
 		} else
 			ex_to<basic>(basis).setflag(status_flags::purely_indefinite);
 	}
@@ -868,26 +833,23 @@ ex power::expand(unsigned options) const
 		const add &a = ex_to<add>(expanded_exponent);
 		exvector distrseq;
 		distrseq.reserve(a.seq.size() + 1);
-		epvector::const_iterator last = a.seq.end();
-		epvector::const_iterator cit = a.seq.begin();
-		while (cit!=last) {
-			distrseq.push_back(power(expanded_basis, a.recombine_pair_to_ex(*cit)));
-			++cit;
+		for (auto & cit : a.seq) {
+			distrseq.push_back(pow(expanded_basis, a.recombine_pair_to_ex(cit)));
 		}
 		
 		// Make sure that e.g. (x+y)^(2+a) expands the (x+y)^2 factor
 		if (ex_to<numeric>(a.overall_coeff).is_integer()) {
 			const numeric &num_exponent = ex_to<numeric>(a.overall_coeff);
-			int int_exponent = num_exponent.to_int();
+			long int_exponent = num_exponent.to_int();
 			if (int_exponent > 0 && is_exactly_a<add>(expanded_basis))
 				distrseq.push_back(expand_add(ex_to<add>(expanded_basis), int_exponent, options));
 			else
-				distrseq.push_back(power(expanded_basis, a.overall_coeff));
+				distrseq.push_back(pow(expanded_basis, a.overall_coeff));
 		} else
-			distrseq.push_back(power(expanded_basis, a.overall_coeff));
+			distrseq.push_back(pow(expanded_basis, a.overall_coeff));
 		
 		// Make sure that e.g. (x+y)^(1+a) -> x*(x+y)^a + y*(x+y)^a
-		ex r = (new mul(distrseq))->setflag(status_flags::dynallocated);
+		ex r = dynallocate<mul>(distrseq);
 		return r.expand(options);
 	}
 	
@@ -896,13 +858,13 @@ ex power::expand(unsigned options) const
 		if (are_ex_trivially_equal(basis,expanded_basis) && are_ex_trivially_equal(exponent,expanded_exponent)) {
 			return this->hold();
 		} else {
-			return (new power(expanded_basis,expanded_exponent))->setflag(status_flags::dynallocated | (options == 0 ? status_flags::expanded : 0));
+			return dynallocate<power>(expanded_basis, expanded_exponent).setflag(options == 0 ? status_flags::expanded : 0);
 		}
 	}
 	
 	// integer numeric exponent
 	const numeric & num_exponent = ex_to<numeric>(expanded_exponent);
-	int int_exponent = num_exponent.to_int();
+	long int_exponent = num_exponent.to_long();
 	
 	// (x+y)^n, n>0
 	if (int_exponent > 0 && is_exactly_a<add>(expanded_basis))
@@ -916,7 +878,7 @@ ex power::expand(unsigned options) const
 	if (are_ex_trivially_equal(basis,expanded_basis) && are_ex_trivially_equal(exponent,expanded_exponent))
 		return this->hold();
 	else
-		return (new power(expanded_basis,expanded_exponent))->setflag(status_flags::dynallocated | (options == 0 ? status_flags::expanded : 0));
+		return dynallocate<power>(expanded_basis, expanded_exponent).setflag(options == 0 ? status_flags::expanded : 0);
 }
 
 //////////
@@ -929,192 +891,9 @@ ex power::expand(unsigned options) const
 // non-virtual functions in this class
 //////////
 
-namespace {  // anonymous namespace for power::expand_add() helpers
-
-/** Helper class to generate all bounded combinatorial partitions of an integer
- *  n with exactly m parts (including zero parts) in non-decreasing order.
- */
-class partition_generator {
-private:
-	// Partitions n into m parts, not including zero parts.
-	// (Cf. OEIS sequence A008284; implementation adapted from Jörg Arndt's
-	// FXT library)
-	struct mpartition2
-	{
-		// partition: x[1] + x[2] + ... + x[m] = n and sentinel x[0] == 0
-		std::vector<int> x;
-		int n;   // n>0
-		int m;   // 0<m<=n
-		mpartition2(unsigned n_, unsigned m_)
-		  : x(m_+1), n(n_), m(m_)
-		{
-			for (int k=1; k<m; ++k)
-				x[k] = 1;
-			x[m] = n - m + 1;
-		}
-		bool next_partition()
-		{
-			int u = x[m];  // last element
-			int k = m;
-			int s = u;
-			while (--k) {
-				s += x[k];
-				if (x[k] + 2 <= u)
-					break;
-			}
-			if (k==0)
-				return false;  // current is last
-			int f = x[k] + 1;
-			while (k < m) {
-				x[k] = f;
-				s -= f;
-				++k;
-			}
-			x[m] = s;
-			return true;
-		}
-	} mpgen;
-	int m;  // number of parts 0<m<=n
-	mutable std::vector<int> partition;  // current partition
-public:
-	partition_generator(unsigned n_, unsigned m_)
-	  : mpgen(n_, 1), m(m_), partition(m_)
-	{ }
-	// returns current partition in non-decreasing order, padded with zeros
-	const std::vector<int>& current() const
-	{
-		for (int i = 0; i < m - mpgen.m; ++i)
-			partition[i] = 0;  // pad with zeros
-
-		for (int i = m - mpgen.m; i < m; ++i)
-			partition[i] = mpgen.x[i - m + mpgen.m + 1];
-
-		return partition;
-	}
-	bool next()
-	{
-		if (!mpgen.next_partition()) {
-			if (mpgen.m == m || mpgen.m == mpgen.n)
-				return false;  // current is last
-			// increment number of parts
-			mpgen = mpartition2(mpgen.n, mpgen.m + 1);
-		}
-		return true;
-	}
-};
-
-/** Helper class to generate all compositions of a partition of an integer n,
- *  starting with the compositions which has non-decreasing order.
- */
-class composition_generator {
-private:
-	// Generates all distinct permutations of a multiset.
-	// (Based on Aaron Williams' algorithm 1 from "Loopless Generation of
-	// Multiset Permutations using a Constant Number of Variables by Prefix
-	// Shifts." <http://webhome.csc.uvic.ca/~haron/CoolMulti.pdf>)
-	struct coolmulti {
-		// element of singly linked list
-		struct element {
-			int value;
-			element* next;
-			element(int val, element* n)
-			  : value(val), next(n) {}
-			~element()
-			{   // recurses down to the end of the singly linked list
-				delete next;
-			}
-		};
-		element *head, *i, *after_i;
-		// NB: Partition must be sorted in non-decreasing order.
-		explicit coolmulti(const std::vector<int>& partition)
-		{
-			head = NULL;
-			for (unsigned n = 0; n < partition.size(); ++n) {
-				head = new element(partition[n], head);
-				if (n <= 1)
-					i = head;
-			}
-			after_i = i->next;
-		}
-		~coolmulti()
-		{   // deletes singly linked list
-			delete head;
-		}
-		void next_permutation()
-		{
-			element *before_k;
-			if (after_i->next != NULL && i->value >= after_i->next->value)
-				before_k = after_i;
-			else
-				before_k = i;
-			element *k = before_k->next;
-			before_k->next = k->next;
-			k->next = head;
-			if (k->value < head->value)
-				i = k;
-			after_i = i->next;
-			head = k;
-		}
-		bool finished() const
-		{
-			return after_i->next == NULL && after_i->value >= head->value;
-		}
-	} cmgen;
-	bool atend;  // needed for simplifying iteration over permutations
-	bool trivial;  // likewise, true if all elements are equal
-	mutable std::vector<int> composition;  // current compositions
-public:
-	explicit composition_generator(const std::vector<int>& partition)
-	  : cmgen(partition), atend(false), trivial(true), composition(partition.size())
-	{
-		for (unsigned i=1; i<partition.size(); ++i)
-			trivial = trivial && (partition[0] == partition[i]);
-	}
-	const std::vector<int>& current() const
-	{
-		coolmulti::element* it = cmgen.head;
-		size_t i = 0;
-		while (it != NULL) {
-			composition[i] = it->value;
-			it = it->next;
-			++i;
-		}
-		return composition;
-	}
-	bool next()
-	{
-		// This ugly contortion is needed because the original coolmulti
-		// algorithm requires code duplication of the payload procedure,
-		// one before the loop and one inside it.
-		if (trivial || atend)
-			return false;
-		cmgen.next_permutation();
-		atend = cmgen.finished();
-		return true;
-	}
-};
-
-/** Helper function to compute the multinomial coefficient n!/(p1!*p2!*...*pk!)
- *  where n = p1+p2+...+pk, i.e. p is a partition of n.
- */
-const numeric
-multinomial_coefficient(const std::vector<int> & p)
-{
-	numeric n = 0, d = 1;
-	std::vector<int>::const_iterator it = p.begin(), itend = p.end();
-	while (it != itend) {
-		n += numeric(*it);
-		d *= factorial(numeric(*it));
-		++it;
-	}
-	return factorial(numeric(n)) / d;
-}
-
-}  // anonymous namespace
-
 /** expand a^n where a is an add and n is a positive integer.
  *  @see power::expand */
-ex power::expand_add(const add & a, int n, unsigned options) const
+ex power::expand_add(const add & a, long n, unsigned options)
 {
 	// The special case power(+(x,...y;x),2) can be optimized better.
 	if (n==2)
@@ -1176,7 +955,7 @@ ex power::expand_add(const add & a, int n, unsigned options) const
 	// i.e. the number of unordered arrangements of m nonnegative integers
 	// which sum up to n.  It is frequently written as C_n(m) and directly
 	// related with binomial coefficients: binomial(n+m-1,m-1).
-	size_t result_size = binomial(numeric(n+a.nops()-1), numeric(a.nops()-1)).to_int();
+	size_t result_size = binomial(numeric(n+a.nops()-1), numeric(a.nops()-1)).to_long();
 	if (!a.overall_coeff.is_zero()) {
 		// the result's overall_coeff is one of the terms
 		--result_size;
@@ -1202,18 +981,18 @@ ex power::expand_add(const add & a, int n, unsigned options) const
 		// Multinomial expansion of power(+(x,...,z;0),k)*c^(n-k):
 		// Iterate over all partitions of k with exactly as many parts as
 		// there are symbolic terms in the basis (including zero parts).
-		partition_generator partitions(k, a.seq.size());
+		partition_with_zero_parts_generator partitions(k, a.seq.size());
 		do {
-			const std::vector<int>& partition = partitions.current();
+			const std::vector<unsigned>& partition = partitions.get();
 			// All monomials of this partition have the same number of terms and the same coefficient.
-			const unsigned msize = count_if(partition.begin(), partition.end(), bind2nd(std::greater<int>(), 0));
+			const unsigned msize = std::count_if(partition.begin(), partition.end(), [](int i) { return i > 0; });
 			const numeric coeff = multinomial_coefficient(partition) * binomial_coefficient;
 
 			// Iterate over all compositions of the current partition.
 			composition_generator compositions(partition);
 			do {
-				const std::vector<int>& exponent = compositions.current();
-				exvector monomial;
+				const std::vector<unsigned>& exponent = compositions.get();
+				epvector monomial;
 				monomial.reserve(msize);
 				numeric factor = coeff;
 				for (unsigned i = 0; i < exponent.size(); ++i) {
@@ -1231,44 +1010,46 @@ ex power::expand_add(const add & a, int n, unsigned options) const
 						// optimize away
 					} else if (exponent[i] == 1) {
 						// optimized
-						monomial.push_back(r);
+						monomial.emplace_back(expair(r, _ex1));
 						if (c != *_num1_p)
 							factor = factor.mul(c);
 					} else { // general case exponent[i] > 1
-						monomial.push_back((new power(r, exponent[i]))->setflag(status_flags::dynallocated));
+						monomial.emplace_back(expair(r, exponent[i]));
 						if (c != *_num1_p)
 							factor = factor.mul(c.power(exponent[i]));
 					}
 				}
-				result.push_back(a.combine_ex_with_coeff_to_pair(mul(monomial).expand(options), factor));
+				result.emplace_back(expair(mul(std::move(monomial)).expand(options), factor));
 			} while (compositions.next());
 		} while (partitions.next());
 	}
 
 	GINAC_ASSERT(result.size() == result_size);
-
 	if (a.overall_coeff.is_zero()) {
-		return (new add(result))->setflag(status_flags::dynallocated |
-		                                  status_flags::expanded);
+		return dynallocate<add>(std::move(result)).setflag(status_flags::expanded);
 	} else {
-		return (new add(result, ex_to<numeric>(a.overall_coeff).power(n)))->setflag(status_flags::dynallocated |
-		                                                                            status_flags::expanded);
+		return dynallocate<add>(std::move(result), ex_to<numeric>(a.overall_coeff).power(n)).setflag(status_flags::expanded);
 	}
 }
 
 
 /** Special case of power::expand_add. Expands a^2 where a is an add.
  *  @see power::expand_add */
-ex power::expand_add_2(const add & a, unsigned options) const
+ex power::expand_add_2(const add & a, unsigned options)
 {
-	epvector sum;
-	size_t a_nops = a.nops();
-	sum.reserve((a_nops*(a_nops+1))/2);
-	epvector::const_iterator last = a.seq.end();
+	epvector result;
+	size_t result_size = (a.nops() * (a.nops()+1)) / 2;
+	if (!a.overall_coeff.is_zero()) {
+		// the result's overall_coeff is one of the terms
+		--result_size;
+	}
+	result.reserve(result_size);
+
+	auto last = a.seq.end();
 
 	// power(+(x,...,z;c),2)=power(+(x,...,z;0),2)+2*c*+(x,...,z;0)+c*c
 	// first part: ignore overall_coeff and expand other terms
-	for (epvector::const_iterator cit0=a.seq.begin(); cit0!=last; ++cit0) {
+	for (auto cit0=a.seq.begin(); cit0!=last; ++cit0) {
 		const ex & r = cit0->rest;
 		const ex & c = cit0->coeff;
 		
@@ -1282,50 +1063,48 @@ ex power::expand_add_2(const add & a, unsigned options) const
 		
 		if (c.is_equal(_ex1)) {
 			if (is_exactly_a<mul>(r)) {
-				sum.push_back(a.combine_ex_with_coeff_to_pair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
-				                                              _ex1));
+				result.emplace_back(expair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
+				                           _ex1));
 			} else {
-				sum.push_back(a.combine_ex_with_coeff_to_pair((new power(r,_ex2))->setflag(status_flags::dynallocated),
-				                                              _ex1));
+				result.emplace_back(expair(dynallocate<power>(r, _ex2),
+				                           _ex1));
 			}
 		} else {
 			if (is_exactly_a<mul>(r)) {
-				sum.push_back(a.combine_ex_with_coeff_to_pair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
-				                     ex_to<numeric>(c).power_dyn(*_num2_p)));
+				result.emplace_back(expair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
+				                           ex_to<numeric>(c).power_dyn(*_num2_p)));
 			} else {
-				sum.push_back(a.combine_ex_with_coeff_to_pair((new power(r,_ex2))->setflag(status_flags::dynallocated),
-				                     ex_to<numeric>(c).power_dyn(*_num2_p)));
+				result.emplace_back(expair(dynallocate<power>(r, _ex2),
+				                           ex_to<numeric>(c).power_dyn(*_num2_p)));
 			}
 		}
 
-		for (epvector::const_iterator cit1=cit0+1; cit1!=last; ++cit1) {
+		for (auto cit1=cit0+1; cit1!=last; ++cit1) {
 			const ex & r1 = cit1->rest;
 			const ex & c1 = cit1->coeff;
-			sum.push_back(a.combine_ex_with_coeff_to_pair(mul(r,r1).expand(options),
-			                                              _num2_p->mul(ex_to<numeric>(c)).mul_dyn(ex_to<numeric>(c1))));
+			result.emplace_back(expair(mul(r,r1).expand(options),
+			                           _num2_p->mul(ex_to<numeric>(c)).mul_dyn(ex_to<numeric>(c1))));
 		}
 	}
 	
-	GINAC_ASSERT(sum.size()==(a.seq.size()*(a.seq.size()+1))/2);
-	
 	// second part: add terms coming from overall_coeff (if != 0)
 	if (!a.overall_coeff.is_zero()) {
-		epvector::const_iterator i = a.seq.begin(), end = a.seq.end();
-		while (i != end) {
-			sum.push_back(a.combine_pair_with_coeff_to_pair(*i, ex_to<numeric>(a.overall_coeff).mul_dyn(*_num2_p)));
-			++i;
-		}
-		sum.push_back(expair(ex_to<numeric>(a.overall_coeff).power_dyn(*_num2_p),_ex1));
+		for (auto & i : a.seq)
+			result.push_back(a.combine_pair_with_coeff_to_pair(i, ex_to<numeric>(a.overall_coeff).mul_dyn(*_num2_p)));
+	}
+
+	GINAC_ASSERT(result.size() == result_size);
+
+	if (a.overall_coeff.is_zero()) {
+		return dynallocate<add>(std::move(result)).setflag(status_flags::expanded);
+	} else {
+		return dynallocate<add>(std::move(result), ex_to<numeric>(a.overall_coeff).power(2)).setflag(status_flags::expanded);
 	}
-	
-	GINAC_ASSERT(sum.size()==(a_nops*(a_nops+1))/2);
-	
-	return (new add(sum))->setflag(status_flags::dynallocated | status_flags::expanded);
 }
 
 /** Expand factors of m in m^n where m is a mul and n is an integer.
  *  @see power::expand */
-ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool from_expand) const
+ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool from_expand)
 {
 	GINAC_ASSERT(n.is_integer());
 
@@ -1353,20 +1132,17 @@ ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool fr
 	distrseq.reserve(m.seq.size());
 	bool need_reexpand = false;
 
-	epvector::const_iterator last = m.seq.end();
-	epvector::const_iterator cit = m.seq.begin();
-	while (cit!=last) {
-		expair p = m.combine_pair_with_coeff_to_pair(*cit, n);
-		if (from_expand && is_exactly_a<add>(cit->rest) && ex_to<numeric>(p.coeff).is_pos_integer()) {
+	for (auto & cit : m.seq) {
+		expair p = m.combine_pair_with_coeff_to_pair(cit, n);
+		if (from_expand && is_exactly_a<add>(cit.rest) && ex_to<numeric>(p.coeff).is_pos_integer()) {
 			// this happens when e.g. (a+b)^(1/2) gets squared and
 			// the resulting product needs to be reexpanded
 			need_reexpand = true;
 		}
 		distrseq.push_back(p);
-		++cit;
 	}
 
-	const mul & result = static_cast<const mul &>((new mul(distrseq, ex_to<numeric>(m.overall_coeff).power_dyn(n)))->setflag(status_flags::dynallocated));
+	const mul & result = dynallocate<mul>(std::move(distrseq), ex_to<numeric>(m.overall_coeff).power_dyn(n));
 	if (need_reexpand)
 		return ex(result).expand(options);
 	if (from_expand)
