@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.5.3.  */
+/* A Bison parser, made by GNU Bison 3.6.3.  */
 
 /* Bison interface for Yacc-like parsers in C
 
@@ -31,8 +31,9 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-/* Undocumented macros, especially those whose name start with YY_,
-   are private implementation details.  Do not rely on them.  */
+/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
+   especially those whose name start with YY_ or yy_.  They are
+   private implementation details that can be changed or removed.  */
 
 #ifndef YY_YY_GINSH_PARSER_HPP_INCLUDED
 # define YY_YY_GINSH_PARSER_HPP_INCLUDED
@@ -44,39 +45,47 @@
 extern int yydebug;
 #endif
 
-/* Token type.  */
+/* Token kinds.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
   enum yytokentype
   {
-    T_NUMBER = 258,
-    T_SYMBOL = 259,
-    T_LITERAL = 260,
-    T_DIGITS = 261,
-    T_QUOTE = 262,
-    T_QUOTE2 = 263,
-    T_QUOTE3 = 264,
-    T_EQUAL = 265,
-    T_NOTEQ = 266,
-    T_LESSEQ = 267,
-    T_GREATEREQ = 268,
-    T_QUIT = 269,
-    T_WARRANTY = 270,
-    T_PRINT = 271,
-    T_IPRINT = 272,
-    T_PRINTLATEX = 273,
-    T_PRINTCSRC = 274,
-    T_TIME = 275,
-    T_XYZZY = 276,
-    T_INVENTORY = 277,
-    T_LOOK = 278,
-    T_SCORE = 279,
-    T_COMPLEX_SYMBOLS = 280,
-    T_REAL_SYMBOLS = 281,
-    NEG = 282
+    YYEMPTY = -2,
+    YYEOF = 0,                     /* "end of file"  */
+    YYerror = 256,                 /* error  */
+    YYUNDEF = 257,                 /* "invalid token"  */
+    T_NUMBER = 258,                /* T_NUMBER  */
+    T_SYMBOL = 259,                /* T_SYMBOL  */
+    T_LITERAL = 260,               /* T_LITERAL  */
+    T_DIGITS = 261,                /* T_DIGITS  */
+    T_QUOTE = 262,                 /* T_QUOTE  */
+    T_QUOTE2 = 263,                /* T_QUOTE2  */
+    T_QUOTE3 = 264,                /* T_QUOTE3  */
+    T_EQUAL = 265,                 /* T_EQUAL  */
+    T_NOTEQ = 266,                 /* T_NOTEQ  */
+    T_LESSEQ = 267,                /* T_LESSEQ  */
+    T_GREATEREQ = 268,             /* T_GREATEREQ  */
+    T_QUIT = 269,                  /* T_QUIT  */
+    T_WARRANTY = 270,              /* T_WARRANTY  */
+    T_PRINT = 271,                 /* T_PRINT  */
+    T_IPRINT = 272,                /* T_IPRINT  */
+    T_PRINTLATEX = 273,            /* T_PRINTLATEX  */
+    T_PRINTCSRC = 274,             /* T_PRINTCSRC  */
+    T_TIME = 275,                  /* T_TIME  */
+    T_XYZZY = 276,                 /* T_XYZZY  */
+    T_INVENTORY = 277,             /* T_INVENTORY  */
+    T_LOOK = 278,                  /* T_LOOK  */
+    T_SCORE = 279,                 /* T_SCORE  */
+    T_COMPLEX_SYMBOLS = 280,       /* T_COMPLEX_SYMBOLS  */
+    T_REAL_SYMBOLS = 281,          /* T_REAL_SYMBOLS  */
+    NEG = 282                      /* NEG  */
   };
+  typedef enum yytokentype yytoken_kind_t;
 #endif
-/* Tokens.  */
+/* Token kinds.  */
+#define YYEOF 0
+#define YYerror 256
+#define YYUNDEF 257
 #define T_NUMBER 258
 #define T_SYMBOL 259
 #define T_LITERAL 260
